;********************************************************************************
; Cypress Semiconductor Corporation
; Cypress’ Source Code: Copyright 2010, Cypress Semiconductor Corporation.
; This Source Code (software and*or firmware) is owned by Cypress Semiconductor
; Corporation (Cypress) and is protected by and subject to worldwide patent 
; protection (United States and foreign), United States copyright laws and 
; international treaty provisions. Cypress hereby grants to licensee a personal,
; non-exclusive, non-transferable license to copy, use, modify, create 
; derivative works of, and compile the Cypress Source Code and derivative works
; for the sole purpose of creating custom software and or firmware in support
; of licensee product to be used only in conjunction with a Cypress integrated
; circuit as specified in the applicable agreement. Any reproduction,
; modification, translation, compilation, or representation of this Source Code
; except as specified above is prohibited without the express written
; permission of Cypress. 
; Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
; REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
; OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
; Cypress reserves the right to make changes without further notice to the
; materials described herein. Cypress does not assume any liability arising
; out of the application or use of any product or circuit described herein.
; Cypress does not authorize its products for use as critical components in
; life-support systems where a malfunction or failure may reasonably be expected
; to result in significant injury to the user. The inclusion of Cypress’ product
; in a life-support systems application implies that the manufacturer assumes
; all risk of such use and in doing so indemnifies Cypress against all charges.
;
; Use may be limited by and subject to the applicable Cypress software license
; agreement.
;
;********************************************************************************
;------------------------------------------------------------------------------
; FILENAME: FlashBlock.asm
; Version: 2.1
;------------------------------------------------------------------------------
; DESCRIPTION:
; Routines in this file perform Flash Block Write and Read operations for Ge
; family.
;------------------------------------------------------------------------------
; Copyright (c) Cypress MicroSystems 2010. All rights reserved.
;------------------------------------------------------------------------------
include "m8c.inc"
include "m8ssc.inc"
include "flashblock.inc"
include "memory.inc"		;File generated by PSoC Designer (Project dependent)

bARG_BlockId: equ 0 ; block ID
pARG_FlashBuffer: equ 1 ; flash buffer pointer - 2 bytes
cARG_Temperature: equ 3 ; flash Tempurature
bDATA_PWErase: equ 4 ; temporary storage (reserved)
bDATA_PWProgram: equ 5 ; temporary storage (reserved)
bDATA_PWMultiplier: equ 6 ; temporary storage (reserved)
FLASH_WRITE_STRUCT_SIZE: equ 7 ; Calling frame size

export bFlashWriteBlock
export _bFlashWriteBlock
export FlashReadBlock
export _FlashReadBlock
export FlashCheckSum
export _FlashCheckSum
export Boot_Is_Program_Good
export _Boot_Is_Program_Good

;--------------------------------------------
; Declare Flash RAM storage at top of RAM
; This data area is reserved for exclusive
; use by Supervisory operations.
;--------------------------------------------

area SSCParmBlk(RAM,ABS) 

org F8H 

SSCParameterBlk: blk 8 ; reserve 8 bytes: F8 - FF for supervisory operation

;--------------------------------

; SSC_Action macro command codes 

;-------------------------------- 

FLASH_OPER_KEY: equ 3Ah ; flash operation key
FLASH_WRITE: equ 2 ; flash write operation supervisory command
FLASH_ERASE: equ 3 ; flash erase operation supervisory command
FLASH_TEMP_TABLE_LOOKUP: equ 6 ; flash temperature table command

;-------------------------------------

; Flash temperature programming table

;-------------------------------------

TEMPERATURE_TABLE_ID: equ 3 ; flash temperature programming table ID


;--------------------------------------------------------------

; Supervisory Operation Templates:

; Each system supervisory call uses the reserved data area

; a little different. Create overlay templates to improve

; readability and maintenance. 

;--------------------------------------------------------------

;Common Supervisory Code Variables

bSSC_KEY1: equ F8h ; supervisory key
bSSC_KEYSP: equ F9h ; supervisory stack ptr key
bSSC_ResultCode: equ F8h ; result code

;SSC Return Codes
bSSC_FAIL: equ 01h ; fail return code
bSSC_SUCCESS: equ 00h ; success return code

;Flash Erase Operation Template
bSSC_FLASH_BlockId: equ FAh ; blockId for Erase and Read
bSSC_FLASH_PulseWidth: equ FCh ; Erase pulse width 

;Flash Write Operation Template
bSSC_WRITE_BlockId: equ FAh ; block ID
pSSC_WRITE_FlashBuffer: equ FBh ; pointer to data buffer
bSSC_WRITE_PulseWidth: equ FCh ; flash write pulse width or ReadCount
bSSC_WRITE_State: equ FDh ; reserved 
bSSC_WRITE_Delay: equ FEh ; flash macro sequence delay count
bSSC_WRITE_ResultCode: equ FFh ; temporary result code

;Flash Write Operation Return Code
bSSC_WRITE_FAIL: equ 00h ; failure
bSSC_WRITE_SUCCESS: equ 01h ; pass

;Flash Sequence Time Delay

bSSC_WRITE_DELAY: equ 56h ; 100us delay at 12MHz

;Flash Read Operation Template

pSSC_READ_FlashBuffer: equ FBh ; pointer to data buffer
bSSC_READ_Counter: equ FDh ; byte counter

; Table Lookup Template - NOTE that result is a table overlaying reserved area

bSSC_TABLE_TableId: equ FAh ; table ID

; Temperature Table Template - returned data after performing a 

; Table lookup of table #7 - Pulse Width Equation data based on Temperature - y= mt+b

; Composed to two structures - 3 Bytes each - contains slope, y intercept and multiplier

; Structure 1 is for temperatures from [-40,0]C and structure 2 is for temperaturs from [0,40]C.
sTEMP_LineEquationBelowZero: equ F8h ; Line Equation data below zero - m,b,Erase->Program multiplier
sTEMP_LineEquationAboveZero: equ FBh ; Line Equation data above zero - m,b,Erase->Program multiplier
bTEMP_TempTableRevision: equ FFh ; Table Revision number
bTEMP_PulseWidthErase: equ FEh ; Temporary storage area after table validation
bTEMP_PulseWidthProgram: equ FFh ; Temporary storage area after table validation

;offsets into each substructure

cTEMP_SlopeOffset: equ 0 ; Slope offset in Temp table template
cTEMP_InterceptOffset: equ 1 ; Intercept offset in Temp table template
bTEMP_ProgMultOffset: equ 2 ; Program multiplier 
VALID_TABLE_REVISION: equ 1 ; Temp Table revision number

 

; Flash State mode bits - these bits are used to prevent inadvertent jumping into

; the flash write API. If the state does not match then a HALT instruction will

; be executed.

STATE_SET_CLOCK: equ 01h
STATE_CALCULATE_PW: equ 02h
STATE_ERASE_BLOCK: equ 04h
STATE_WRITE_BLOCK: equ 08h
STATE_RESTORE_CLOCK: equ 10h
STATE_DONE: equ 20h

IF	(TOOLCHAIN & HITECH)
psect BootLoaderArea global
ELSE
Area BootLoaderArea(ROM,REL,CON)
ENDIF
;-----------------------------------------------------------------------------
; MACRO NAME: SSC_Action
;
; DESCRIPTION:
; Performs locally defined supervisory operations.
;
; !!! DO NOT CHANGE THIS CODE !!!
; This sequence of opcodes provides a 
; signature for the debugger and ICE.
; !!! DO NOT CHANGE THIS CODE !!!
;
; ARGUMENTS:
; BYTE bOperation - specified supervisory operation - defined operations
; are: FLASH_WRITE, FLASH_ERASE, FLASH_TEMP_TABLE_LOOKUP.
;
; RETURNS:
; none.
;
; SIDE EFFECTS:
; A and X registers are destroyed
;
; PROCEDURE: 
; 1) specify a 3 byte stack frame. Save in [KEYSP]
; 2) insert the flash Supervisory key in [KEY1]
; 3) store flash operation function code in A
; 4) call the supervisory code
;-----------------------------------------------------------------------------

macro SSC_Action 
	mov X, SP ; copy SP into X
	mov A, X ; mov to A
	add A, 3 ; create 3 byte stack frame
	mov [bSSC_KEYSP], A ; save stack frame for supervisory code
	mov [bSSC_KEY1], FLASH_OPER_KEY ; load the supervisory code for flash operations
	mov A, @0 ; load A with specific Flash operation
	SSC ; SSC call the supervisory code
endm

// ----------------------------------------------------------------------------------------
//	Control subroutine which determine whether user program is based on BootLoader Project
//  Return: 0 - if bad program
// 	  	    1 - if all right
// ----------------------------------------------------------------------------------------
Boot_Is_Program_Good:
_Boot_Is_Program_Good:
	push X
	mov A,[__r0]
	push A
	
	mov A,0
	mov X,A0h			; init start control sequence address
	mov [__r0],A		; initiate counter
Start_Control:	
	mov A,0
	romx
	cmp A,[__r0]		; does addr 00A0 contain 0x7D (ljump)
	jnz Error_Program
	inc [__r0]
	inc X
	mov A,X
	cmp A,A6h			; is > than last sequence address?
	jnz Start_Control	; goto next sequence address
	
	pop A	
	mov [__r0],A
	pop X
	mov A,1		; restore register from stack
	ret			; if ok then return 1
Error_Program:
	pop A	
	mov [__r0],A
	pop X		; restore register from stack
	mov A,0		; if bad program then return 0
	ret	

// ---------------------------
// Flash Block CheckSum
// Entrance: A - block ID
// Exit: A - CheckSum
FlashCheckSum:
_FlashCheckSum:

	push X
	mov X,SP

	mov A,[__r0]
	push A
	mov A,[__r1]
	push A
	mov A,[__r2]
	push A

	mov A,[X-4]						; A = BlockID
					// calculate address addr = (BlockID>>2)||(BlockID<<6) = BlockID*64
	push A		; save BlockID
	asl A
	asl A
	asl A
	asl A
	asl A
	asl A
	mov X,A		; save Address MSB = (BlockID<<6) in X
	pop A		; get BlockID
	asr A
	asr A		; save Address MSB = (BlockID>>2)	in A
	
	mov [__r1],A		; Address MSB
	mov [__r0],64		; Cycle Iterator
	mov [__r2],0		; CheckSum
calc_cs:
	dec [__r0]			; decrement Cycle Iterator
	mov A,[__r1]
	romx
	add [__r2],A
	inc X				; increment X to get next address
	or [__r0],0			; if ([__r0]!=0) goto calc_cs;
	jnz calc_cs			; 

	mov A,[__r2]		; return CheckSum  
	
	pop X
	mov [__r2],X
	pop X
	mov [__r1],X
	pop X
	mov [__r0],X

//	mov [bSSC_WRITE_BlockId],A
//	SSC_Action FLASH_CHECKSUM		// CheckSum return in [KEY2][KEY1]
//	mov A,[bSSC_KEY1]		
	pop X
	ret

;-----------------------------------------------------------------------------
; FUNCTION NAME: bFlashWriteBlock
;
; DESCRIPTION:
; Writes 64 bytes of data to the flash at the specified blockId.
; 
; Regardless of the size of the buffer, this routine always writes 64 
; bytes of data. If the buffer is less than 64 bytes, then the next
; 64-N bytes of data will be written to fill the rest of flash block data. 
;
; ARGUMENTS:
; X -> psBlockWriteData - a structure that holds the 
; calling arguments and some reserved space 
;
; RETURNS:
; BYTE - successful if NON-Zero returned. 
; 
; ASSEMBLER - returned in Accumulator.
;
; SIDE EFFECTS:
; 1) CPU clock temporarily set to 12MHz.
;
; PROCEDURE:
; 1) Setup the proper CPU clock - 12 MHz
; 2) Compute the pulsewidths
; 3) Erase the specified block
; 4) Program the specified block
; 5) restore the original CPU rate
; 6) check the result code and return
;-----------------------------------------------------------------------------

; Place Halt instruction code here to mitigate wondering into this code from the top

	halt 
	
bFlashWriteBlock:
_bFlashWriteBlock: 

; Set the intial state variable - if code entered from the top - then state
; variable will not catch this inadvertent entry. However, any entry from
; made after this statement should be caught!

	mov [bSSC_WRITE_State], STATE_SET_CLOCK 

; Step 1 - setup the proper CPU clock - 12 MHz
; Check the state variable - are we supposed to be here?

	cmp [bSSC_WRITE_State], STATE_SET_CLOCK
	jnz bFlashWriteStateError

; State - Set Clock

	M8C_SetBank1 
	mov A, reg[OSC_CR0] ; Get the System Oscillator control register

	push A ; and save it on the stack
	and A, ~OSC_CR0_CPU ; Clear the CPU clock selects
	or A, OSC_CR0_CPU_12MHz ; Set CPU clock to 12 MHz 
	mov reg[OSC_CR0], A 
	M8C_SetBank0

; Step 2 - compute the pulsewidths

ComputePulseWidths:

	asl [bSSC_WRITE_State] ; update the state variable

; Check the state variable - are we supposed to be here?

	cmp [bSSC_WRITE_State], STATE_CALCULATE_PW
        jnz bFlashWriteStateError

; State - Calculate PW

	call bComputePulseWidth

; Preset the resturn code to Success

	cmp A, bSSC_SUCCESS ; Check the return value 
        jz EraseBlock ; the pulse width was computed OK
	
; A bad pulse width table was found!

; Need to set the result code, restore the clock, and then exit!

	mov [bSSC_WRITE_ResultCode], bSSC_WRITE_FAIL 
	mov [bSSC_WRITE_State], STATE_RESTORE_CLOCK 
        jmp RestoreClock

; Step 3 - Erase the specified flash block

EraseBlock:

	asl [bSSC_WRITE_State] ; update the state variable

; State - Erase Block

	mov A, [X+bARG_BlockId] ; load ERASE operation parameters
        mov [bSSC_WRITE_BlockId], A 
        mov A, [X+bDATA_PWErase]
        mov [bSSC_WRITE_PulseWidth], A
        mov [bSSC_WRITE_Delay], bSSC_WRITE_DELAY ; load the sequence delay count

; Check the state variable - are we supposed to be here?

	mov A, [bSSC_WRITE_State]
        cmp A, STATE_ERASE_BLOCK
        jnz bFlashWriteStateError
        push A ; save the State variable
        push X
        or reg[PRT0DR],01h

; SSC_Action FLASH_ERASE ; Erase the specified block
        mov X, SP ; copy SP into X
        mov A, X ; mov to A
        add A, 3 ; create 3 byte stack frame
        mov [bSSC_KEYSP], A ; save stack frame for supervisory code
        mov [bSSC_KEY1], FLASH_OPER_KEY ; load the supervisory code for flash operations
        mov A, FLASH_ERASE ; load A with specific Flash operation
        SSC 
        and reg[PRT0DR],~01h
        pop X
        pop A ; restore the State variable
        mov [bSSC_WRITE_State], A

; Check the return code for success
        cmp [bSSC_ResultCode], bSSC_SUCCESS
        jz WriteBlock

; Erase operation failed!
; Need to set the result code, restore the clock, and then exit!
        mov [bSSC_WRITE_ResultCode], bSSC_WRITE_FAIL 
        mov [bSSC_WRITE_State], STATE_RESTORE_CLOCK 
        jmp RestoreClock

; Step 4 - Program the flash

WriteBlock:

; Update the state variable
        asl [bSSC_WRITE_State] 

; State - Write Block
        mov A, [X+bARG_BlockId] ; load WRITE opeation parameters
        mov [bSSC_WRITE_BlockId], A
        mov A, [X+pARG_FlashBuffer+1]
        mov [pSSC_WRITE_FlashBuffer], A
        mov A, [X+bDATA_PWProgram]
        mov [bSSC_WRITE_PulseWidth], A
        mov [bSSC_WRITE_Delay], bSSC_WRITE_DELAY ; load the sequence delay count

; Check the state variable - are we supposed to be here?
        mov A, [bSSC_WRITE_State]
        cmp A, STATE_WRITE_BLOCK
        jnz bFlashWriteStateError
        push A ; save the State variable
        push X
        or reg[PRT0DR],02h
        SSC_Action FLASH_WRITE ; Program the flash
        and reg[PRT0DR],~02h
        pop X
        pop A ; restore the state variable
        mov [bSSC_WRITE_State], A

; Set the return code 
        mov [bSSC_WRITE_ResultCode], bSSC_WRITE_SUCCESS
        cmp [bSSC_ResultCode], bSSC_SUCCESS
        jz NextState

; Write operation failed!

; Need to set the result code, restore the clock, and then exit!
        mov [bSSC_WRITE_ResultCode], bSSC_WRITE_FAIL 

; Step 5 - restore the original CPU rate

NextState:
        asl [bSSC_WRITE_State] ; update the state variable

; Check the state variable - are we supposed to be here?

RestoreClock:
        cmp [bSSC_WRITE_State], STATE_RESTORE_CLOCK
        jnz bFlashWriteStateError

; State - Restore Clock
        pop A
        M8C_SetBank1
        mov reg[OSC_CR0], A ; Restore org CPU rate
        M8C_SetBank0 ; Switch back to Bank 0
        asl [bSSC_WRITE_State] ; update the state variable

; Step 6 - Compute the return result code
        mov A, [bSSC_WRITE_ResultCode] 

bFlashWriteBlockEnd:

; check the state variable for proper exit - 

cmp [bSSC_WRITE_State], STATE_DONE

jz bFlashWriteExit

; if we arrived here, it means that the flashWrite API was randomly entered!!!
bFlashWriteStateError:
        halt
        jmp bFlashWriteStateError

bFlashWriteExit:
        ret

; Put halt here in case we jump inadvertently
        halt
        halt
        halt

;-----------------------------------------------------------------------------
; FUNCTION NAME: ComputePulseWidth
;
; DESCRIPTION:
; Computes the Block Erase and Block Program pulse width counts for the
; Flash Erase and Flash Program supervisory calls.
;
; This routine gets its data from the FlashWriteBlock data structure
; and saves the return data in the same structure.
;
; First, the Temperature data table is accessed via the Table Read SSC
; function. Then the Erase and Program pulse width counts are computed.
;
; Temperature table gives the slope, Y intercept, and Erase to Program pulse
; width converion. Two equations are given - temperatures below 0 and 
; temperatures above 0. Data is scaled to fit within specified byte range. 
;
; PW(erase) = B - M*T*2/256 and PW(program)= PW(erase)*Multiplier/64
;
; ARGUMENTS:
; X points to bFlashWriteBlock calling structure.
;
; RETURNS:
; BYTE bResult - return in Accumulator
; 0 = valid
; 1 = invalid revision
;
; Erase and Program pulse widths are returned in bFlashWriteBlock calling
; structure.
;
; SIDE EFFECTS:
; none.
;
; PROCEDURE:
; 1) Get the flash programming temperature table
; 2) Check the table revision number
; 3) Select the correct data set, based on temperature 
; 4) Compute the Erase Pulsewidth count
; 5) Compute the Program Pulsewidth count
; 6) Save the result data
; 7) If Erase Pulse width is > 127 then adjust PWProgram
;
;-----------------------------------------------------------------------------

bComputePulseWidth:

; 1) Get the flash programming temperature table

	mov [bSSC_TABLE_TableId], TEMPERATURE_TABLE_ID

; Check the State

	mov A, [bSSC_WRITE_State]
        cmp A, STATE_CALCULATE_PW
        jnz bFlashWriteStateError
        push A ; save the State variable
        push X
        SSC_Action FLASH_TEMP_TABLE_LOOKUP
        pop X

; 2) Check table revision
        cmp [bTEMP_TempTableRevision], VALID_TABLE_REVISION
        jnz ComputePulseWidthTableError ; jump if revision is out of date!

; 3) Select the correct data set, based on temperature
        push X
        mov A, [X+cARG_Temperature] ; load temperature into the MULTIPLIER
        mov REG[MUL_X], A
        and A, 80h ; Test for sign of temperature
        jnz BelowZero 

AboveZero: ; Positive temperature
        mov A, [sTEMP_LineEquationAboveZero + bTEMP_ProgMultOffset]
        mov [X+bDATA_PWMultiplier], A ; Save the multiplier for later use
        mov X, sTEMP_LineEquationAboveZero ; X pts to Temp table above zero
        jmp ComputeData

BelowZero: ; Negative temperature
        mov A, [sTEMP_LineEquationBelowZero + bTEMP_ProgMultOffset]
        mov [X+bDATA_PWMultiplier], A ; Save the multiplier for later use
        mov X, sTEMP_LineEquationBelowZero ; X pts to Temp table data below zero

; 4) Compute the Erase PulseWidth count => PW(e) = B - M*T*2/256

ComputeData:
        mov A, [X+cTEMP_SlopeOffset] ; compute M*T
        mov REG[MUL_Y], A
        mov A, REG[MUL_DL] ; compute M*T*2
        asl A 
        mov A, REG[MUL_DH]
        rlc A ; A = M*T*2/256
        cpl A ; 2's complement the data - complement and then increment
        inc A ; A = -(M*T*2/256)
        add A, [X+cTEMP_InterceptOffset] ; Add it to B to compute PW(e) => B - (M*T*2/256) => ERASE PulseWidth
        mov [bTEMP_PulseWidthErase], A ; HACK Save the Erase Pulse width in temp area

; 5) Compute the Program PulseWidth
        and A, 7Fh ; Mac is signed - if >127 then strip off MSbit - taken care of in 7)
        mov REG[MUL_X], A ; compute PW(e) * ProgMultiplier
        mov A, [X+bTEMP_ProgMultOffset]
        mov REG[MUL_Y], A 
        mov A, REG[MUL_DL] ; compute PW(e) * ProgMultiplier / 64
        mov [bTEMP_PulseWidthProgram], A 
        mov A, REG[MUL_DH] 
        asl [bTEMP_PulseWidthProgram] 
        rlc A
        asl [bTEMP_PulseWidthProgram]
        rlc A ; A = PW(erase) * ProgramMultiplier / 64

; 6) Save the result data
        pop X 
        mov [X+bDATA_PWProgram], A ; Save Program pulse width in BlockWrite calling frame
        mov A, [bTEMP_PulseWidthErase] ; Save Erase pulse width in BlockWrite calling frame
        mov [X+bDATA_PWErase], A ; HACK

; 7) If Erase Pulse width is greater then 127 then add 128*ProgMultiplier to PWProgram
        tst [bTEMP_PulseWidthErase], 80h
        jz ComputePulseWidthEnd
        mov A, [X+bDATA_PWMultiplier]
        asl A
        add [X+bDATA_PWProgram], A

ComputePulseWidthEnd: ; NORMAL Termination
        pop A ; restore the STATE
        cmp A, STATE_CALCULATE_PW ; make sure we are supposed to be here
        jnz bFlashWriteStateError
        mov [bSSC_WRITE_State], A ; restore the state variable
        mov A, bSSC_SUCCESS ; load return value with success!
        ret

ComputePulseWidthTableError: ; TABLE Error Termination
        pop A ; restore the STATE
        cmp A, STATE_CALCULATE_PW ; make sure we are supposed to be here
        jnz bFlashWriteStateError
        mov [bSSC_WRITE_State], A ; restore the state variable
        mov A, bSSC_FAIL ; load return value with Failure!
        ret

;------------------------------------------------------------------------------
; FUNCTION NAME: FlashReadBlock
;
; DESCRIPTION:
; Reads a specified flash block to a buffer in RAM.
;
; ARGUMENTS:
; X -> FLASH_READ_STRUCT
; 
; RETURNS:
; Data read is returned at specified pFlashBuffer.
;
; SIDE EFFECTS:
; Uses SSC storage at FBh and FDh
;
; PROCEDURE: 
; BlockID is converted to absolute address and then ROMX command is used
; to read the data from flash into the specified buffer.
;
;------------------------------------------------------------------------------

FlashReadBlock:

_FlashReadBlock: 

; mov some args to SSC storage
	mov A, [X+pARG_FlashBuffer+1] ; get pointer
        mov [pSSC_READ_FlashBuffer],A ; use SSC storage area
        mov A, [X+bARG_ReadCount] ; get count
        mov [bSSC_READ_Counter],A ; use SSC storage area

; Compute the absolute address of the flash block
        mov A, [X+bARG_BlockId] ; compute the LSB = bBlockId * 64
        rrc A
        rrc A
        rrc A
        and A, C0h
        push A ; save LSB on stack
        mov A, [X+bARG_BlockId] ; compute the MSB = bBlockId / 4
        asr A
        asr A
        and A, 3Fh ; mask off sign extension bits
        pop X ; LSB in X, MSB in A

; Read the Flash
ReadFlash: 
        push A ; save MSB
        romx ; Read the flash
        mvi [pSSC_READ_FlashBuffer], A ; store the data in the RAM buffer
        pop A ; restore MSB
        inc X ; increment the LSB of the flash addr
        dec [bSSC_READ_Counter] ; decrement the byte counter
        jnz ReadFlash
        
; Done reading the flash
        ret
;---------------------
; End of File
;---------------------
