;********************************************************************************
; Cypress Semiconductor Corporation
; Cypress’ Source Code: Copyright 2010, Cypress Semiconductor Corporation.
; This Source Code (software and*or firmware) is owned by Cypress Semiconductor
; Corporation (Cypress) and is protected by and subject to worldwide patent 
; protection (United States and foreign), United States copyright laws and 
; international treaty provisions. Cypress hereby grants to licensee a personal,
; non-exclusive, non-transferable license to copy, use, modify, create 
; derivative works of, and compile the Cypress Source Code and derivative works
; for the sole purpose of creating custom software and or firmware in support
; of licensee product to be used only in conjunction with a Cypress integrated
; circuit as specified in the applicable agreement. Any reproduction,
; modification, translation, compilation, or representation of this Source Code
; except as specified above is prohibited without the express written
; permission of Cypress. 
; Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
; REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
; OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
; Cypress reserves the right to make changes without further notice to the
; materials described herein. Cypress does not assume any liability arising
; out of the application or use of any product or circuit described herein.
; Cypress does not authorize its products for use as critical components in
; life-support systems where a malfunction or failure may reasonably be expected
; to result in significant injury to the user. The inclusion of Cypress’ product
; in a life-support systems application implies that the manufacturer assumes
; all risk of such use and in doing so indemnifies Cypress against all charges.
;
; Use may be limited by and subject to the applicable Cypress software license
; agreement.
;
;********************************************************************************

include "m8c.inc"
include "GlobalParams.inc"
include "bootloader.inc"
include "memory.inc"		;File generated by PSoC Designer (Project dependent)


IF	(TOOLCHAIN & HITECH)
psect BootLoaderArea global
ELSE
Area BootLoaderArea(ROM,REL,CON)
ENDIF

export   Boot_RxD_Start
export  _Boot_RxD_Start
export   Boot_RxD_Stop
export  _Boot_RxD_Stop
export   Boot_RxD_bReadRxData
export  _Boot_RxD_bReadRxData
export   Boot_RxD_bReadRxStatus
export  _Boot_RxD_bReadRxStatus

export   Boot_TxD_Start
export  _Boot_TxD_Start
export   Boot_TxD_Stop
export  _Boot_TxD_Stop
export   Boot_TxD_SendData
export  _Boot_TxD_SendData
export   Boot_TxD_bReadTxStatus
export  _Boot_TxD_bReadTxStatus

export  Boot_Counter_EnableInt
export _Boot_Counter_EnableInt
export  Boot_Counter_DisableInt
export _Boot_Counter_DisableInt
export  Boot_Counter_Start
export _Boot_Counter_Start
export  Boot_Counter_Stop
export _Boot_Counter_Stop
export  Boot_Counter_WritePeriod
export _Boot_Counter_WritePeriod
export  Boot_Counter_WriteCompareValue
export _Boot_Counter_WriteCompareValue
export  Boot_Counter_wReadCompareValue
export _Boot_Counter_wReadCompareValue
export  Boot_Counter_wReadCounter
export _Boot_Counter_wReadCounter

//---------------------------------------------------------------------------------
// Configuration
//---------------------------------------------------------------------------------
export Boot_LoadConfigInit
export _Boot_LoadConfigInit

//export NO_SHADOW
//export _NO_SHADOW

FLAG_CFG_MASK:		equ	10h			;M8C flag register REG address bit mask
END_CONFIG_TABLE:	equ	ffh			;end of config table indicator	

Boot_LoadConfigInit:
_Boot_LoadConfigInit:
	
	lcall	LoadConfigTBL_BootLoaderConfig_Ordered
	lcall	LoadConfig_BootLoaderConfig

	ret

;
; Load Configuration BootLoaderConfig
;
_LoadConfig_BootLoaderConfig:
LoadConfig_BootLoaderConfig:
    push    a
    push    x
	M8C_SetBank0									;switch to bank 0
	mov		a, 0
	asr		a
	mov		A, >LoadConfigTBL_BootLoaderConfig_Bank0	;load bank 0 table
	mov		X, <LoadConfigTBL_BootLoaderConfig_Bank0
	lcall	LoadConfig								;load the bank 0 values
	M8C_SetBank1									;set for bank 1
	mov		a, 1
	asr		a
	mov		A, >LoadConfigTBL_BootLoaderConfig_Bank1	;load bank 1 table
	mov		X, <LoadConfigTBL_BootLoaderConfig_Bank1
	lcall	LoadConfig								;load the bank 1 values
	M8C_SetBank0									;switch to bank 0
    pop     x
    pop     a
	ret
;
; LoadConfig
;
;  This function is not exported.  It assumes that the address of the table
;  to be loaded is contained in the X and A registers as if a romx instruction
;  is the next instruction to be executed, i.e. lower address in X and uppper
;  address in A.  There is no return value.
;
LoadConfig:
	add		SP, 2					;set up temp vars
	push	X
	push	A
	mov		X, SP
	mov		[X-4], 0
	jnc		LoadBank0Setup
	mov		[X-4], 1
LoadBank0Setup:
	pop		A
	pop		X
LoadConfigLp:
	push	X						;save config table address on stack
	push	A
	M8C_SetBank0					;switch to bank 0
	M8C_ClearWDT					;clear the watchdog for long inits
	mov		X, SP					;check for bank 1 load
	tst		[X-4], 1
	jz		LoadingBank0
	M8C_SetBank1
LoadingBank0:
	pop		A
	pop		X
	push	X
	push	A
	romx							;load config address
	cmp		A, END_CONFIG_TABLE		;check for end of table
	jz		EndLoadConfig			;if so, end of load
	mov		X, SP					;save the address away
	mov		[X-3], A
	pop		A						;retrieve the table address
	pop		X
	inc		X						;advance to the data byte
	jnc		NoOverFlow1				;check for overflow
	inc		A						;if so, increment MSB
NoOverFlow1:	
	push	X						;save the config table address again
	push	A
	romx							;load the config data
	mov		X, SP					;retrieve the config address
	mov		X, [X-3]
	mov		reg[X], A				;write the config data
	pop		A						;retrieve the table address
	pop		X
	inc		X						;advance to the next address
	jnc		NoOverFlow2				;check for overflow
	inc		A						;if so, increment MSB
NoOverFlow2:	
	jmp		LoadConfigLp			;loop back
EndLoadConfig:
	pop		A						;clean up the stack
	pop		X
	add		SP, -2
	ret

NO_SHADOW:
_NO_SHADOW:

LoadConfigTBL_BootLoaderConfig_Ordered:
;  Ordered Global Register values
	M8C_SetBank0
	mov	reg[00h], 00h		; Port_0_Data register (PRT0DR)
	M8C_SetBank1
	mov	reg[00h], 00h		; Port_0_DriveMode_0 register (PRT0DM0)
	mov	reg[01h], ffh		; Port_0_DriveMode_1 register (PRT0DM1)
	M8C_SetBank0
	mov	reg[03h], ffh		; Port_0_DriveMode_2 register (PRT0DM2)
	mov	reg[02h], 00h		; Port_0_GlobalSelect register (PRT0GS)
	M8C_SetBank1
	mov	reg[02h], 00h		; Port_0_IntCtrl_0 register (PRT0IC0)
	mov	reg[03h], 00h		; Port_0_IntCtrl_1 register (PRT0IC1)
	M8C_SetBank0
	mov	reg[01h], 00h		; Port_0_IntEn register (PRT0IE)
	mov	reg[04h], 00h		; Port_1_Data register (PRT1DR)
	M8C_SetBank1
	mov	reg[04h], 40h		; Port_1_DriveMode_0 register (PRT1DM0)
	mov	reg[05h], bfh		; Port_1_DriveMode_1 register (PRT1DM1)
	M8C_SetBank0
	mov	reg[07h], bfh		; Port_1_DriveMode_2 register (PRT1DM2)
	mov	reg[06h], 00h		; Port_1_GlobalSelect register (PRT1GS)
	M8C_SetBank1
	mov	reg[06h], 00h		; Port_1_IntCtrl_0 register (PRT1IC0)
	mov	reg[07h], 00h		; Port_1_IntCtrl_1 register (PRT1IC1)
	M8C_SetBank0
	mov	reg[05h], 00h		; Port_1_IntEn register (PRT1IE)
	mov	reg[08h], 00h		; Port_2_Data register (PRT2DR)
	M8C_SetBank1
	mov	reg[08h], 02h		; Port_2_DriveMode_0 register (PRT2DM0)
	mov	reg[09h], fdh		; Port_2_DriveMode_1 register (PRT2DM1)
	M8C_SetBank0
	mov	reg[0bh], fch		; Port_2_DriveMode_2 register (PRT2DM2)
	mov	reg[0ah], 03h		; Port_2_GlobalSelect register (PRT2GS)
	M8C_SetBank1
	mov	reg[0ah], 00h		; Port_2_IntCtrl_0 register (PRT2IC0)
	mov	reg[0bh], 00h		; Port_2_IntCtrl_1 register (PRT2IC1)
	M8C_SetBank0
	mov	reg[09h], 00h		; Port_2_IntEn register (PRT2IE)
	mov	reg[0ch], 00h		; Port_3_Data register (PRT3DR)
	M8C_SetBank1
	mov	reg[0ch], 00h		; Port_3_DriveMode_0 register (PRT3DM0)
	mov	reg[0dh], 00h		; Port_3_DriveMode_1 register (PRT3DM1)
	M8C_SetBank0
	mov	reg[0fh], 00h		; Port_3_DriveMode_2 register (PRT3DM2)
	mov	reg[0eh], 00h		; Port_3_GlobalSelect register (PRT3GS)
	M8C_SetBank1
	mov	reg[0eh], 00h		; Port_3_IntCtrl_0 register (PRT3IC0)
	mov	reg[0fh], 00h		; Port_3_IntCtrl_1 register (PRT3IC1)
	M8C_SetBank0
	mov	reg[0dh], 00h		; Port_3_IntEn register (PRT3IE)
	mov	reg[10h], 00h		; Port_4_Data register (PRT4DR)
	M8C_SetBank1
	mov	reg[10h], 00h		; Port_4_DriveMode_0 register (PRT4DM0)
	mov	reg[11h], 00h		; Port_4_DriveMode_1 register (PRT4DM1)
	M8C_SetBank0
	mov	reg[13h], 00h		; Port_4_DriveMode_2 register (PRT4DM2)
	mov	reg[12h], 00h		; Port_4_GlobalSelect register (PRT4GS)
	M8C_SetBank1
	mov	reg[12h], 00h		; Port_4_IntCtrl_0 register (PRT4IC0)
	mov	reg[13h], 00h		; Port_4_IntCtrl_1 register (PRT4IC1)
	M8C_SetBank0
	mov	reg[11h], 00h		; Port_4_IntEn register (PRT4IE)
	mov	reg[14h], 00h		; Port_5_Data register (PRT5DR)
	M8C_SetBank1
	mov	reg[14h], 00h		; Port_5_DriveMode_0 register (PRT5DM0)
	mov	reg[15h], 00h		; Port_5_DriveMode_1 register (PRT5DM1)
	M8C_SetBank0
	mov	reg[17h], 00h		; Port_5_DriveMode_2 register (PRT5DM2)
	mov	reg[16h], 00h		; Port_5_GlobalSelect register (PRT5GS)
	M8C_SetBank1
	mov	reg[16h], 00h		; Port_5_IntCtrl_0 register (PRT5IC0)
	mov	reg[17h], 00h		; Port_5_IntCtrl_1 register (PRT5IC1)
	M8C_SetBank0
	mov	reg[15h], 00h		; Port_5_IntEn register (PRT5IE)
	M8C_SetBank0
	ret
	
LoadConfigTBL_BootLoaderConfig_Bank0:
;  Global Register values
;  Instance name Boot_Counter, User Module Counter16
;       Instance name Boot_Counter, Block Name CNTR16_LSB(DBB00)
	db		23h, 00h		;Boot_Counter_CONTROL_LSB_REG(DBB00CR0)
	db		21h, 00h		;Boot_Counter_PERIOD_LSB_REG(DBB00DR1)
	db		22h, 00h		;Boot_Counter_COMPARE_LSB_REG(DBB00DR2)
;       Instance name Boot_Counter, Block Name CNTR16_MSB(DBB01)
	db		27h, 00h		;Boot_Counter_CONTROL_MSB_REG(DBB01CR0)
	db		25h, 80h		;Boot_Counter_PERIOD_MSB_REG(DBB01DR1)
	db		26h, 40h		;Boot_Counter_COMPARE_MSB_REG(DBB01DR2)
;  Instance name Boot_RxD, User Module RX8
;       Instance name Boot_RxD, Block Name RX8(DCB03)
	db		2fh, 00h		;Boot_RxD_CONTROL_REG  (DCB03CR0)
	db		2dh, 00h		;Boot_RxD_(DCB03DR1)
	db		2eh, 00h		;Boot_RxD_RX_BUFFER_REG(DCB03DR2)
;  Instance name Boot_TxD, User Module TX8
;       Instance name Boot_TxD, Block Name TX8(DCB02)
	db		2bh, 00h		;Boot_TxD_CONTROL_REG  (DCB02CR0)
	db		29h, 00h		;Boot_TxD_TX_BUFFER_REG(DCB02DR1)
	db		2ah, 00h		;Boot_TxD_(DCB02DR2)
;  Global Register values Bank 0
	db		60h, 28h		; AnalogColumnInputSelect register (AMX_IN)
	db		66h, 00h		; AnalogComparatorControl1 register (CMP_CR1)
	db		63h, 05h		; AnalogReferenceControl register (ARF_CR)
	db		65h, 00h		; AnalogSyncControl register (ASY_CR)
	db		e6h, 00h		; DecimatorControl_0 register (DEC_CR0)
	db		e7h, 00h		; DecimatorControl_1 register (DEC_CR1)
	db		d6h, 00h		; I2CConfig register (I2CCFG)
	db		b0h, 00h		; Row_0_InputMux register (RDI0RI)
	db		b1h, 0fh		; Row_0_InputSync register (RDI0SYN)
	db		b2h, 00h		; Row_0_LogicInputAMux register (RDI0IS)
	db		b3h, 33h		; Row_0_LogicSelect_0 register (RDI0LT0)
	db		b4h, 33h		; Row_0_LogicSelect_1 register (RDI0LT1)
	db		b5h, 10h		; Row_0_OutputDrive_0 register (RDI0SRO0)
	db		b6h, 00h		; Row_0_OutputDrive_1 register (RDI0SRO1)
	db		b8h, 55h		; Row_1_InputMux register (RDI1RI)
	db		b9h, 0fh		; Row_1_InputSync register (RDI1SYN)
	db		bah, 10h		; Row_1_LogicInputAMux register (RDI1IS)
	db		bbh, 33h		; Row_1_LogicSelect_0 register (RDI1LT0)
	db		bch, 33h		; Row_1_LogicSelect_1 register (RDI1LT1)
	db		bdh, 00h		; Row_1_OutputDrive_0 register (RDI1SRO0)
	db		beh, 00h		; Row_1_OutputDrive_1 register (RDI1SRO1)
	db		ffh

LoadConfigTBL_BootLoaderConfig_Bank1:
;  Instance name Boot_Counter, User Module Counter16
;       Instance name Boot_Counter, Block Name CNTR16_LSB(DBB00)
	db		20h, 11h		;Boot_Counter_FUNC_LSB_REG(DBB00FN)
	db		21h, 17h		;Boot_Counter_INPUT_LSB_REG(DBB00IN)
	db		22h, 40h		;Boot_Counter_OUTPUT_LSB_REG(DBB00OU)
;       Instance name Boot_Counter, Block Name CNTR16_MSB(DBB01)
	db		24h, 31h		;Boot_Counter_FUNC_MSB_REG(DBB01FN)
	db		25h, 37h		;Boot_Counter_INPUT_MSB_REG(DBB01IN)
	db		26h, 40h		;Boot_Counter_OUTPUT_MSB_REG(DBB01OU)
;  Instance name Boot_RxD, User Module RX8
;       Instance name Boot_RxD, Block Name RX8(DCB03)
	db		2ch, 05h		;Boot_RxD_FUNC_REG     (DCB03FN)
	db		2dh, c1h		;Boot_RxD_INPUT_REG    (DCB03IN)
	db		2eh, 40h		;Boot_RxD_OUTPUT_REG   (DCB03OU)
;  Instance name Boot_TxD, User Module TX8
;       Instance name Boot_TxD, Block Name TX8(DCB02)
	db		28h, 0dh		;Boot_TxD_FUNC_REG     (DCB02FN)
	db		29h, 01h		;Boot_TxD_INPUT_REG    (DCB02IN)
	db		2ah, 45h		;Boot_TxD_OUTPUT_REG   (DCB02OU)
;  Global Register values Bank 1
	db		61h, 00h		; AnalogClockSelect1 register (CLK_CR1)
	db		69h, 00h		; AnalogClockSelect2 register (CLK_CR2)
	db		60h, 00h		; AnalogColumnClockSelect register (CLK_CR0)
	db		62h, 00h		; AnalogIOControl_0 register (ABF_CR0)
	db		67h, 33h		; AnalogLUTControl0 register (ALT_CR0)
	db		68h, 33h		; AnalogLUTControl1 register (ALT_CR1)
	db		63h, 00h		; AnalogModulatorControl_0 register (AMD_CR0)
	db		66h, 00h		; AnalogModulatorControl_1 register (AMD_CR1)
	db		d1h, 00h		; GlobalDigitalInterconnect_Drive_Even_Input register (GDI_E_IN)
	db		d3h, 00h		; GlobalDigitalInterconnect_Drive_Even_Output register (GDI_E_OU)
	db		d0h, 00h		; GlobalDigitalInterconnect_Drive_Odd_Input register (GDI_O_IN)
	db		d2h, 00h		; GlobalDigitalInterconnect_Drive_Odd_Output register (GDI_O_OU)
	db		e1h, ffh		; OscillatorControl_1 register (OSC_CR1)
	db		e2h, 00h		; OscillatorControl_2 register (OSC_CR2)
	db		dfh, 19h		; OscillatorControl_3 register (OSC_CR3)
	db		deh, 00h		; OscillatorControl_4 register (OSC_CR4)
	db		ddh, 00h		; OscillatorGlobalBusEnableControl register (OSC_GO_EN)
	db		ffh

//---------------------------------------------------
//---------------------------------------------------
//		User Modules
//---------------------------------------------------
//---------------------------------------------------

//---------------------------------------------------
// Boot_RxD
//---------------------------------------------------
bfCONTROL_REG_START_BIT:   equ   1     ; Control register start bit 


.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Boot_RxD_Start(BYTE bParity)
;
;  DESCRIPTION:
;    Sets the start bit and parity in the Control register of this user module. 
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;    BYTE bParity - parity of received data.  Use defined masks.
;    passed in A register.
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;     Set the specified parity and start bits in the Control register.
;
;-----------------------------------------------------------------------------
 Boot_RxD_Start:
_Boot_RxD_Start:
   or    A, bfCONTROL_REG_START_BIT
   mov   REG[Boot_RxD_CONTROL_REG], A
   ret	
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Boot_RxD_Stop
;
;  DESCRIPTION:
;     Disables RX8 operation.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: none
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;     Clear the start bit in the Control register.
;
;-----------------------------------------------------------------------------
 Boot_RxD_Stop:
_Boot_RxD_Stop:
   and   REG[Boot_RxD_CONTROL_REG], ~bfCONTROL_REG_START_BIT
   ret	
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Boot_RxD_bReadRxData
;
;  DESCRIPTION:
;     Reads the RX buffer register.  Should check the status regiser to make
;     sure data is valid.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:  none
;
;  RETURNS:
;    bRxData - returned in A.
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
 Boot_RxD_bReadRxData:
_Boot_RxD_bReadRxData:
 	mov A, REG[Boot_RxD_RX_BUFFER_REG]
	ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Boot_RxD_bReadRxStatus
;
;  DESCRIPTION:
;    Reads the RX Status bits in the Control/Status register.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:  none
;
;  RETURNS:
;     BYTE  bRXStatus - transmit status data.  Use the following defined bits 
;                       masks: RX_COMPLETE and RX_BUFFER_EMPTY
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;     Read the status and control register.
;
;-----------------------------------------------------------------------------
 Boot_RxD_bReadRxStatus:
_Boot_RxD_bReadRxStatus:
	mov A,  REG[Boot_RxD_CONTROL_REG]
	ret
.ENDSECTION



//-----------------------------------------------------
// Boot_TxD
//-----------------------------------------------------
;  FUNCTION NAME: Boot_TxD_Start(BYTE bParity)
;
;  DESCRIPTION:
;     Sets the start bit and parity in the Control register of this user module. 
;     The transmitter will begin transmitting if a byte has been written into the
;     transmit buffer.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;    BYTE bParity - parity of transmitted data.  Use defined masks.
;
;  RETURNS:  none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE:  A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;     Set the specified parity and start bits in the Control register.
;
;-----------------------------------------------------------------------------
 Boot_TxD_Start:
_Boot_TxD_Start:
   or    A, bfCONTROL_REG_START_BIT
   mov   REG[Boot_TxD_CONTROL_REG], A
   ret	



;-----------------------------------------------------------------------------
;  FUNCTION NAME: Boot_TxD_Stop
;
;  DESCRIPTION:
;     Disables TX8 operation.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:  none
;
;  RETURNS:  none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE:  A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;     Clear the start bit in the Control register.
;
;-----------------------------------------------------------------------------
 Boot_TxD_Stop:
_Boot_TxD_Stop:
   and   REG[Boot_TxD_CONTROL_REG], ~bfCONTROL_REG_START_BIT
   ret	


;-----------------------------------------------------------------------------
;  FUNCTION NAME: Boot_TxD_SendData
;
;  DESCRIPTION:
;     Sends one byte through serial port.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     BYTE  TxData - data to transmit.
;
;  RETURNS:
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE:  A AND X REGISTERS MAY BE MODIFIED!
;
;-----------------------------------------------------------------------------
 Boot_TxD_SendData:
_Boot_TxD_SendData:
	mov REG[Boot_TxD_TX_BUFFER_REG], A
	ret

;-----------------------------------------------------------------------------
;  FUNCTION NAME: Boot_TxD_bReadTxStatus
;
;  DESCRIPTION:
;     Reads the Tx Status bits in the Control/Status register.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;     BYTE  bTxStatus - transmit status data.  Use the following defined bits 
;                       masks: TX_COMPLETE and TX_BUFFER_EMPTY
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE:  A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;     Read the status and control register.
;
;-----------------------------------------------------------------------------
 Boot_TxD_bReadTxStatus:
_Boot_TxD_bReadTxStatus:
	mov A,  REG[Boot_TxD_CONTROL_REG]
	ret
   


INPUT_REG_NULL:                equ 0x00    ; Clear the input register

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Boot_Counter_EnableInt
;
;  DESCRIPTION:
;     Enables this counter's interrupt by setting the interrupt enable mask bit
;     associated with this User Module. This function has no effect until and
;     unless the global interrupts are enabled (for example by using the
;     macro M8C_EnableGInt).
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None.
;  RETURNS:      Nothing.
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Boot_Counter_EnableInt:
_Boot_Counter_EnableInt:
   Boot_Counter_EnableInt_M
   ret

.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Boot_Counter_DisableInt
;
;  DESCRIPTION:
;     Disables this counter's interrupt by clearing the interrupt enable
;     mask bit associated with this User Module.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None
;  RETURNS:      Nothing
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Boot_Counter_DisableInt:
_Boot_Counter_DisableInt:
   Boot_Counter_DisableInt_M
   ret


.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Boot_Counter_Start
;
;  DESCRIPTION:
;     Sets the start bit in the Control register of this user module.  The
;     counter will begin counting on the next input clock as soon as the
;     enable input is asserted high.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None
;  RETURNS:      Nothing
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Boot_Counter_Start:
_Boot_Counter_Start:
   Boot_Counter_Start_M
   ret


.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Boot_Counter_Stop
;
;  DESCRIPTION:
;     Disables counter operation by clearing the start bit in the Control
;     register of the LSB block.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None
;  RETURNS:      Nothing
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Boot_Counter_Stop:
_Boot_Counter_Stop:
   Boot_Counter_Stop_M
   ret


.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Boot_Counter_WritePeriod
;
;  DESCRIPTION:
;     Write the 16-bit period value into the Period register (DR1).
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: fastcall WORD wPeriodValue (LSB in A, MSB in X)
;  RETURNS:   Nothing
;  SIDE EFFECTS:
;     If the counter user module is stopped, then this value will also be
;     latched into the Count registers (DR0).
;     REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Boot_Counter_WritePeriod:
_Boot_Counter_WritePeriod:
   mov   reg[Boot_Counter_PERIOD_LSB_REG], A
   mov   A, X
   mov   reg[Boot_Counter_PERIOD_MSB_REG], A
   ret


.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Boot_Counter_WriteCompareValue
;
;  DESCRIPTION:
;     Writes compare value into the Compare register (DR2).
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    fastcall WORD wCompareValue (LSB in A, MSB in X)
;  RETURNS:      Nothing
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Boot_Counter_WriteCompareValue:
_Boot_Counter_WriteCompareValue:
   mov   reg[Boot_Counter_COMPARE_LSB_REG], A
   mov   A, X
   mov   reg[Boot_Counter_COMPARE_MSB_REG], A
   ret


.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Boot_Counter_wReadCompareValue
;
;  DESCRIPTION:
;     Reads the Compare registers.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None
;  RETURNS:      fastcall WORD wCompareValue (value of DR2 in the X & A registers)
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Boot_Counter_wReadCompareValue:
_Boot_Counter_wReadCompareValue:
 wBoot_Counter_ReadCompareValue:                 ; this name deprecated
_wBoot_Counter_ReadCompareValue:                 ; this name deprecated
   mov   A, reg[Boot_Counter_COMPARE_MSB_REG]
   mov   X, A
   mov   A, reg[Boot_Counter_COMPARE_LSB_REG]
   ret


.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Boot_Counter_wReadCounter
;
;  DESCRIPTION:
;     Returns the value in the Count register (DR0), preserving the value in
;     the compare register (DR2). Interrupts are prevented during the transfer
;     from the Count to the Compare registers by holding the clock low in
;     the MSB PSoC block.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: None
;  RETURNS:   fastcall WORD wCount (value of DR0 in the X & A registers)
;  SIDE EFFECTS:
;     1) The user module is stopped momentarily and one or more counts may be missed.
;     2) REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Boot_Counter_wReadCounter:
_Boot_Counter_wReadCounter:
 wBoot_Counter_ReadCounter:                      ; this name deprecated
_wBoot_Counter_ReadCounter:                      ; this name deprecated

   bOrigCompareValue:      EQU   0                  ; Frame offset to temp Compare store
   bOrigControlReg:        EQU   2                  ; Frame offset to temp CR0     store
   bOrigClockSetting:      EQU   3                  ; Frame offset to temp Input   store
   wCounter:               EQU   4                  ; Frame offset to temp Count   store
   STACK_FRAME_SIZE:       EQU   6                  ; max stack frame size is 6 bytes

   mov   X, SP                                      ; X <-  stack frame pointer
   mov   A, reg[Boot_Counter_COMPARE_MSB_REG]    ; Save the Compare register on the stack
   push  A                                          ;
   mov   A, reg[Boot_Counter_COMPARE_LSB_REG]    ;
   push  A                                          ;   -stack frame now 2 bytes-
   mov   A, reg[Boot_Counter_CONTROL_LSB_REG]    ; Save CR0 (running or stopped state)
   push  A                                          ;   -stack frame now 3 bytes-
   Boot_Counter_Stop_M                           ; Disable (stop) the Counter if running
   M8C_SetBank1                                     ;
   mov   A, reg[Boot_Counter_INPUT_LSB_REG]      ; save the LSB clock input setting
   push  A                                          ;   on the stack (now 4 bytes) and ...
                                                    ;   hold the clock low:
   mov   reg[Boot_Counter_INPUT_LSB_REG], INPUT_REG_NULL
   M8C_SetBank0                                     ; Extract the Count via DR2 register
   mov   A, reg[Boot_Counter_COUNTER_MSB_REG]    ; DR2 <- DR0 (in the MSB block)
   mov   A, reg[Boot_Counter_COMPARE_MSB_REG]    ; Stash the Count MSB on the stack
   push  A                                          ;   -stack frame is now 5 bytes
   mov   A, reg[Boot_Counter_COUNTER_LSB_REG]    ; DR2 <- DR0 (in the LSB block)
   mov   A, reg[Boot_Counter_COMPARE_LSB_REG]    ; Stash the Count LSB on the stack
   push  A                                          ;   -stack frame is now 6 bytes-
   mov   A, [X+bOrigCompareValue]                   ; Restore the Compare MSB register
   mov   reg[Boot_Counter_COMPARE_MSB_REG], A    ;
   mov   A, [X+bOrigCompareValue+1]                 ; Restore the Compare LSB register
   mov   reg[Boot_Counter_COMPARE_LSB_REG], A    ;
   M8C_SetBank1                                     ; ---Restore the counter operation
   mov   A, [X+bOrigClockSetting]                   ; Grab the LSB clock setting...
   mov   reg[Boot_Counter_INPUT_LSB_REG], A      ;   and restore it
   M8C_SetBank0                                     ; Now re-enable (start) the counter
   mov   A, [X+bOrigControlReg]                     ;   if it was running when
   mov   reg[Boot_Counter_CONTROL_LSB_REG], A    ;   this function was first called
   pop   A                                          ; Setup the return value
   pop   X                                          ;
   ADD   SP, -(STACK_FRAME_SIZE-2)                  ; Zap remainder of stack frame
   ret

.ENDSECTION
; End of File Boot_Counter.asm